@@ -1,7 +1,18 @@
 #!/usr/bin/python
 
 '''
-KEGG-decoder.py V.1.2
+KEGG-decoder.py V.1.3
+V.1.3
+Added several pathways associated with carotenoid biosynthesis, including
+end-products: astaxanthin, nostoxanthin, zeaxanthin diglucoside, & 
+myxoxanthophylls. Plus, staphyloaxanthin biosynthesis and the two pathways for
+terpenoid building blocks, the mevalonate pathway and the MEP/DOXP pathway.
+The pathways were provided by Dr. Tania Kurbessoian
+V.1.2.1
+Fixed typo in determing reverse TCA cycle as identified by KEGG-Decoder
+user Cheng. Added all-trans-8'-apo-beta-carotenal 15,15'-oxygenase 
+which will cleave apo-carotenals to generate retinal. Suggested by Eric Webb.
+Upstream pathway unknown
 V.1.2
 Added several new pathways including PET degradation, carbon storage,
 related to starch/gylcogen & polyhydroxybutyrate, and posphate storage,
@@ -263,7 +274,7 @@ def reverse_tca(ko_match):
 	if ('K15230' in ko_match and 'K15231' in ko_match):
 		out_data['rTCA Cycle'] = 1
 #citryl-CoA synthetase AND citryl-CoA lyase
-	if ('K15232' in ko_match and 'K15232' in ko_match and 'K15234' in ko_match):
+	if ('K15232' in ko_match and 'K15233' in ko_match and 'K15234' in ko_match):
 		out_data['rTCA Cycle'] = 1
 	return out_data
 
@@ -300,7 +311,7 @@ def wood_ljungdahl(ko_match):
 def three_prop(ko_match):
 	total = 0
 #pyruvate ferredoxin oxidoreductase alpha and beta subunits
-	if ('K00168' in ko_match and 'K00170' in ko_match):
+	if ('K00169' in ko_match and 'K00170' in ko_match):
 		total +=1
 #pyruvate dikinase
 	if ('K01006' in ko_match or 'K01007' in ko_match):
@@ -901,7 +912,7 @@ def oxidative_phoshorylation(ko_match):
 def photosynthesis(ko_match):
 	out_data = {'Photosystem II':0, 'Photosystem I':0, 'Cytochrome b6/f complex':0,
 	"anoxygenic type-II reaction center":0, "anoxygenic type-I reaction center":0,
-	'Retinal biosynthesis':0}
+	'Retinal biosynthesis':0, 'Retinal from apo-carotenals': 0}
 	psII = ['K02703', 'K02706', 'K02705', 'K02704', 'K02707', 'K02708']
 #Photosystem II core complex
 	for i in psII:
@@ -935,9 +946,14 @@ def photosynthesis(ko_match):
 	for i in retinal:
 		if i in ko_match:
 			out_data['Retinal biosynthesis'] += 0.25
+#Retinal production from apo-carotenals
+	if ('K00464' in ko_match):
+		out_data['Retinal from apo-carotenals'] = 1
 	return out_data
 
 
+
+
 def entnerdoudoroff(ko_match):
 	value = 0
 #H6PD; hexose-6-phosphate dehydrogenase
@@ -1433,6 +1449,46 @@ def phosphate_storage(ko_match):
 
     return out_data
 
+def carotenoids(ko_match):
+	out_data = {'carotenoids backbone biosynthesis':0, 'end-product astaxanthin':0,
+	'end-product nostoxanthin':0, 'end-product zeaxanthin diglucoside':0, 
+	'end-product myxoxanthophylls': 0, 'staphyloaxanthin biosynthesis':0,
+	'mevalonate pathway': 0, 'MEP-DOXP pathway':0}
+
+	staphyloaxanthin = ['K10208', 'K10209', 'K10210', 'K00128',
+	'K10211', 'K10212']
+	for i in staphyloaxanthin:
+		if i in ko_match:
+			out_data['staphyloaxanthin biosynthesis'] += 0.167
+	mevalonate = ['K01641', 'K00054', 'K00869','K00938', 'K01597']
+	for i in mevalonate:
+		if i in ko_match:
+			out_data['mevalonate pathway'] += 0.2
+	mepdoxp = ['K01662', 'K00099', 'K00991', 'K00919', 'K01770',
+	'K03526', 'K03527']
+	for i in mepdoxp:
+		if i in ko_match:
+			out_data['MEP-DOXP pathway'] += 0.142
+	backbone = ['K23155', 'K02291', 'K02291', 'K00514']
+	for i in backbone:
+		if i in ko_match:
+			out_data['carotenoids backbone biosynthesis'] += 0.2
+	if ('K06443' in ko_match) or ('K14606' in ko_match):
+		out_data['carotenoids backbone biosynthesis'] += 0.2
+	if ('K02294' in ko_match):
+		out_data['end-product nostoxanthin'] = 1
+	if ('K02294' in ko_match) and ('K14596' in ko_match):
+		out_data['end-product zeaxanthin diglucoside'] = 1
+	if ('K09836' in ko_match) or ('K02292' in ko_match):
+		out_data['end-product astaxanthin'] = 1
+	myxoxanthophylls = ['K08977', 'K02294', 'K00721']
+	for i in myxoxanthophylls:
+		if i in ko_match:
+			out_data['end-product myxoxanthophylls'] += 0.33
+
+	return out_data
+
+
 
 def default_viz(genome_df, outfile_name):
 	import seaborn as sns
@@ -1536,7 +1592,7 @@ def main():
 	'dimethylamine/trimethylamine dehydrogenase',
 	'Photosystem II', 'Photosystem I', 'Cytochrome b6/f complex',
 	'anoxygenic type-II reaction center', 'anoxygenic type-I reaction center',
-	'Retinal biosynthesis',
+	'Retinal biosynthesis', 'Retinal from apo-carotenals',
 	'Entner-Doudoroff Pathway', 'Mixed acid: Lactate', 'Mixed acid: Formate',
 	'Mixed acid: Formate to CO2 & H2', 'Mixed acid: Acetate',
 	'Mixed acid: Ethanol, Acetate to Acetylaldehyde',
@@ -1564,7 +1620,10 @@ def main():
 	'methionine', 'phenylalanine', 'isoleucine', 'leucine',
 	'tryptophan', 'tyrosine', 'aspartate', 'glutamate', 'PET degradation',
 	'starch/glycogen synthesis', 'starch/glycogen degradation', 'polyhydroxybutyrate synthesis',
-	'bidirectional polyphosphate']
+	'bidirectional polyphosphate', 'carotenoids backbone biosynthesis', 
+	'end-product astaxanthin', 'end-product nostoxanthin', 'end-product zeaxanthin diglucoside', 
+	'end-product myxoxanthophylls', 'staphyloaxanthin biosynthesis',
+	'mevalonate pathway', 'MEP-DOXP pathway']
 
 
 	filehandle = str(arg_dict['output'])
@@ -1612,6 +1671,7 @@ def main():
 		pathway_data.update(plastic(genome_data[k]))
 		pathway_data.update(carbon_storage(genome_data[k]))
 		pathway_data.update(phosphate_storage(genome_data[k]))
+		pathway_data.update(carotenoids(genome_data[k]))
 	#    print k, pathway_data
 
 		out_string = str(k)+"\t"
